define(['event','class'], function(Event,Class) {
	var keyNames = {
		32: 'space',
		38: 'up',
		40: 'down',
		39: 'right',
		37: 'left'
	};
	var i;
	for(i=0;i<27;i++) {
		keyNames[i+65] = String.fromCharCode(i+97);
	}

	return Class(Event,{
		keys: {},
		mouse: {
			over: false,
			x: 0,
			y: 0,
			buttons: {}
		},
		constructor: function(canvas) {
			var me = this;
			this.width = canvas.width;
			this.height = canvas.height;
			this.canvas = canvas;
			this.context = canvas.getContext('2d');
			canvas.addEventListener('keyup', function(event) {
				var keyName = keyNames[event.keyCode];
				if (keyName) {
					delete me.keys[keyName];
					me.emit('keyup', keyName);
				}
			},true);
			canvas.addEventListener('keydown', function(event) {
				var keyName = keyNames[event.keyCode];
				if (keyName) {
					me.keys[keyName] = true;
					me.emit('keydown', keyName);
				}
			},true);
			canvas.addEventListener('mouseup', function(event) {
				me.mouse.x = event.offsetX;
				me.mouse.y = event.offsetY;
				delete me.mouse.buttons[event.button];
				me.emit('mouseup', event.button);
			},true);
			canvas.addEventListener('mousedown', function(event) {
				me.mouse.x = event.offsetX;
				me.mouse.y = event.offsetY;
				me.mouse.buttons[event.button] = true;
				me.emit('mousedown', event.button);
			},true);
			canvas.addEventListener('mousemove', function(event) {
				me.mouse.x = event.offsetX;
				me.mouse.y = event.offsetY;
				me.emit('mousemove', event.offsetX, event.offsetY);
			},true);
		},
		start: function(fps) {
			if (this._interval) { throw 'Already started'; }
			fps = fps || 60;
			var me = this;
			this.canvas.setAttribute('tabIndex', '0');
			this.canvas.focus();
			this._interval = setInterval(function() {
				me.emit('update', 1/fps);
				me.context.clearRect(0,0,me.canvas.width,me.canvas.height);
				me.emit('draw', me.context);
			}, 1000/fps);
		},
		stop: function() {
			clearInterval(this._interval);
			delete this['_interval'];
		}
	});
});
